import java.awt.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;

import com.sun.java.swing.*;
import com.sun.java.swing.preview.*;

import sv.util.*;
import sv.ui.*;
import sv.drawer.*;
import sv.kernel.*;

/** 
 * GFPConnect.java
 * Define a frame to draw the graph
 * @version 1.0    10/1/97
 * @author Byeongseob Ki
 */

class GFPConnect extends GFPlane {

  /**
   * constructor
   * @param title title of this class
   * @param dataSet dataSet for this class
   * @param svServer svserver
   */
  public GFPConnect(String title, DataSetCon dataSet, 
                            CPlane config, svserver svServer) {
     super(title, dataSet, config, svServer);

     // layout others     
     setLayout(new BorderLayout());
     canvas = new GCPConnect(this);

     canvas.dh = new DrawHandle(timeArr[currentTime], canvas.getSize());
     DHV.addElement(canvas.dh);

     boolean b = false;
     if (config.colormap != null) {
         b = readColorMap(config.colormap);

         if (b) {
           float value;

           for (int i=0; i<101; i++) {
              value = rspline.computePoint((float)i/100f); 
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              rcolors[i] = value;

              value = gspline.computePoint((float)i/100f);    
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              gcolors[i] = value;

              value = bspline.computePoint((float)i/100f);    
              if (value > 1.0f) value = 1.0f;
              else if (value < 0.0f) value = 0.0f;
              bcolors[i] = value;
           }
         }
     }

     if (config.colormap == null || b == false) {
         int nControls = 5;
         float[] xcontrols = { 0.0f, 0.25f, 0.5f, 0.75f, 1.0f };
         float[] ycontrols = { 0.0f, 0.5f, 1.0f, 0.5f, 0.0f };
         rspline = new NewtonMethod(xcontrols, ycontrols);
         hspline = new NewtonMethod(xcontrols, ycontrols);

         for (int i=0; i<5; i++)
           ycontrols[i] = 0.5f;
         gspline = new NewtonMethod(xcontrols, ycontrols);
         sspline = new NewtonMethod(xcontrols, ycontrols);

         ycontrols[0] = 1.0f;
         ycontrols[1] = 0.5f; 
         ycontrols[2] = 0.0f;
         ycontrols[3] = 0.5f;
         ycontrols[4] = 1.0f;
         bspline = new NewtonMethod(xcontrols, ycontrols);
         lspline = new NewtonMethod(xcontrols, ycontrols);
     }
 
     add("Center", canvas);
     add("South", monitor = new SimpleMonitor());
     monitor.display(dataSet.getTitle() + " created.", false);
  }   


  /**
   * action for boolean arguments
   */
  public void doAction_1(int id, String arg) {
     boolean b;
     if (arg.equals("true"))
        b = true;
     else
        b = false;

     switch (id) {
      case CSPlane.DISPLAYLBL:
          labelMI.setState(b);
          config.display_label = labelMI.getState();

          if (config.display_label) {
             clabelMI.setEnabled(true);
             labelmMI.setEnabled(true);
          } else {
             clabelMI.setEnabled(false);
             labelmMI.setEnabled(false);
          }
            
          canvas.changeNotify();
          break;

      case CSPlane.TICKMARK:
          tickerMI.setState(b);
          config.tickmark = tickerMI.getState();

          if (config.tickmark) 
             ctickerMI.setEnabled(true);
          else
             ctickerMI.setEnabled(false);

          canvas.changeNotify();
          break;

      case CSPlane.GMINMAX:
          checkMI.setState(b);
          config.global_mode = b;
          globalFlag = checkMI.getState();
          if (globalFlag)
              calcGlobalMinMax();
          canvas.minmaxArr = getMinMax();
          break;

      case CSPlane.DISPLAYTIME:
          timeMI.setState(b);
          config.display_time = timeMI.getState();
          break;


      case CSPlane.COLORBYINFO:
          cinfoMI.setState(b);
          if (cinfoMI.getState()) {
            for (Enumeration e = dataSetVector.elements(); 
                                             e.hasMoreElements(); ) {
               DataSetCon dc = (DataSetCon)e.nextElement();
               for (int j=0; j<dc.getnoOfTime(); j++)
                  ((TimeDataCon)(dc.timeData[j])).calcColors();
            }
          }
          config.color_by_info = cinfoMI.getState();
          ((GCPConnect)canvas).destroyImage();
          break;

      case CSPlane.FILTEROVERLAY:
          overlayMI.setState(b);
          config.overlay_mode = overlayMI.getState();
 
          if (config.overlay_mode) {
             monitor.display("Filter overlay mode is on.", true);
             discardMI.setState(false);
          } else {
             monitor.display("Filter overlay mode is off.", true);
             discardMI.setState(true);
          }
          config.discard_mode = discardMI.getState(); 
          return;

      case CSPlane.FILTERDISCARD:
          discardMI.setState(b);
          config.discard_mode = discardMI.getState();
 
          if (config.discard_mode) {
             monitor.display("Filter discard mode is on.", true);
             overlayMI.setState(false);
          } else {
             monitor.display("Filter discard mode is off.", true);
             overlayMI.setState(true);
          }
          config.overlay_mode = overlayMI.getState();
          return;


     }

     canvas.repaint();
  }

  public void doAction_2(int id, String args[]) {
     int x, y, w, h;

     switch (id) {
      case CSPlane.BACKCOLOR:
          int r = Integer.valueOf(args[0]).intValue();
          int g = Integer.valueOf(args[1]).intValue();
          int b = Integer.valueOf(args[2]).intValue();
          config.bgColor = new Color(r, g, b);
          break;

      case CSPlane.RESIZEPDA:
          x = Integer.valueOf(args[0]).intValue();
          y = Integer.valueOf(args[1]).intValue();
          w = Integer.valueOf(args[2]).intValue();
          h = Integer.valueOf(args[3]).intValue();
          canvas.setPDA(new Rectangle(x, y, w, h));
          break;

      case CSPlane.LABELMOVE:
          config.xLabelPoint.x = Integer.valueOf(args[0]).intValue();
          config.xLabelPoint.y = Integer.valueOf(args[1]).intValue();
          config.yLabelPoint.x = Integer.valueOf(args[2]).intValue();
          config.yLabelPoint.y = Integer.valueOf(args[3]).intValue();
          break;

      case CSPlane.TICKERCHANGE:
          int ints[] = { 
              Integer.valueOf(args[0]).intValue(), 
              Integer.valueOf(args[1]).intValue(),  
              Integer.valueOf(args[2]).intValue(), 
              Integer.valueOf(args[3]).intValue(), 
              Integer.valueOf(args[4]).intValue(),
              Integer.valueOf(args[5]).intValue(), 
              Integer.valueOf(args[6]).intValue(), 
              Integer.valueOf(args[7]).intValue(), 
              Integer.valueOf(args[8]).intValue(), 
              Integer.valueOf(args[9]).intValue() };

          boolean bools[] = new boolean[2];
          if (args[10].equals("true"))
              bools[0] = true;
          else
              bools[0] = false;

          if (args[11].equals("true"))
              bools[1] = true;
          else
              bools[1] = false;

          canvas.axis.setSetting(ints, bools);
          break;
      }

      canvas.repaint();
  }


  /**
   * set color map
   */
  public void setColorMap(String[] str) {
     super.setColorMap(str);
     ((GCPConnect)canvas).destroyImage();
     canvas.repaint();
  }       


  /** 
   * get Previous DataSet (for Animation)
   */
  public void previous() {
     ((GCPConnect)canvas).destroyImage();
     super.previous();
  }

  /** 
   * get Next DataSet (for Animation)
   */
  public void next(boolean done) {
     ((GCPConnect)canvas).destroyImage();
     super.next(done);
  }


  // write all objects of this frame to a file
  public void writeToFile(String fname) {
     DataOutputStream os = null;
     byte[] byteArr;
     int sz = dataSetVector.size();
   
     try {
        os = new DataOutputStream(
               new BufferedOutputStream(new FileOutputStream(fname)));
     } catch (Exception e) {
        System.out.println("File writing error(1d CONNECTIVITY) ...");
        return;
     }

     try {
        // write dimension
        os.writeInt(101);    // connection list identification(UGLY!!)

        // write frame title
        byteArr = frameTitle.getBytes();
        os.writeInt(byteArr.length); 
        os.write(byteArr, 0, byteArr.length);
              
        // write configuration stream
        byteArr = config.getFileSettings().getBytes();
        os.writeInt(byteArr.length); 
        os.write(byteArr, 0, byteArr.length);

        // write drawing objects stream
        String drawingobjects = getDHVString();
        if (drawingobjects != null) {
           byteArr = getDHVString().getBytes();
           os.writeInt(byteArr.length); 
           os.write(byteArr, 0, byteArr.length);
        } else 
           os.writeInt(0); 

        // write the number of data sets
        os.writeInt(sz);  

        // now write the data sets
        for (int i=0; i<sz; i++) {
           DataSetCon dataSet = (DataSetCon)dataSetVector.elementAt(i);

           byteArr = dataSet.getTitle().getBytes();
           os.writeInt(byteArr.length); 
           os.write(byteArr, 0, byteArr.length);

           int noOfTimeData = dataSet.timeData.length;
           os.writeInt(noOfTimeData);   // send number of TimeData
           for (int k=0; k<noOfTimeData; k++) 
              dataSet.timeData[k].sendingData(os);
        } 

        os.close();
     } catch (Exception e) {
        System.out.println("Can't write the connection data to a file ...");
     }     

  }


  // make menu system
  protected void makeMenu() {
 
     MenuBar menubar = new MenuBar();
     CSPlane command;
     MenuItem mi;
     int filter_command = 1000;

     // File manager
     Menu fileMenu = new Menu("File");

     addMI("Save as ...", fileMenu, new CSPlane(this, CSPlane.SAVEAS));
     addMI("Print", fileMenu, new CSPlane(this, CSPlane.PRINT));
     addMI("GIF Sequences", fileMenu, 
                              new CSPlane(this, CSPlane.GIFSEQUENCE));
     fileMenu.addSeparator();
     addMI("Quit C-q", fileMenu, new CSPlane(this, CSPlane.QUIT));

     menubar.add(fileMenu);

     // Cut, Copy and Paste
     Menu editMenu = new Menu("Edit");
     addMI("Cut", editMenu, new CSPlane(this, CSPlane.CUT));
     addMI("Copy", editMenu, new CSPlane(this, CSPlane.COPY));
     addMI("Paste", editMenu, new CSPlane(this, CSPlane.PASTE));

     menubar.add(editMenu);

     // Configuration
     Menu configMenu = new Menu("View");

     Menu colorMenu = new Menu("Colors");

     cinfoMI = new CheckboxMenuItem("Gouraud Shading");
     command = new CSPlane(this, CSPlane.COLORBYINFO);
     cinfoMI.addItemListener(command);
     cinfoMI.setState(config.color_by_info);
     colorMenu.add(cinfoMI);

     addMI("Colormap Editor", colorMenu, new CSPlane(this, CSPlane.COLORMAP));

     mi = new MenuItem("Background Color");
     ActionListener bgColorChooser = new ActionListener() {
	public void actionPerformed(ActionEvent e) {
           Color c = 
              JColorChooser.showDialog(GFPConnect.this, 
                                      "Background Color", config.bgColor);
           if (c == null)
               return;
           String args[] = { String.valueOf(c.getRed()), 
                             String.valueOf(c.getGreen()), 
                             String.valueOf(c.getBlue()) };
           svServer.fmanager.action1d(GFPConnect.this, 
                                    CSPlane.BACKCOLOR, args, true);
	}
     };
     mi.addActionListener(bgColorChooser);
     colorMenu.add(mi);

     configMenu.add(colorMenu);

     Menu labelMenu = new Menu("Labels");
     labelMI = new CheckboxMenuItem("Display Labels");
     command = new CSPlane(this, CSPlane.DISPLAYLBL);
     labelMI.addItemListener(command);
     labelMI.setState(config.display_label);
     labelMenu.add(labelMI);

     clabelMI = new MenuItem("Change Labels");
     command = new CSPlane(this, CSPlane.LABELCHANGE);
     clabelMI.addActionListener(command);
     labelMenu.add(clabelMI);

     labelmMI = new MenuItem("Move Labels");
     command = new CSPlane(this, CSPlane.LABELMOVE);
     labelmMI.addActionListener(command);
     labelMenu.add(labelmMI);

     if (config.display_label) {
        clabelMI.setEnabled(true);
        labelmMI.setEnabled(true);
     } else {
        clabelMI.setEnabled(false);
        labelmMI.setEnabled(false);
     }

     configMenu.add(labelMenu);

     timeMI = new CheckboxMenuItem("Display Time");
     command = new CSPlane(this, CSPlane.DISPLAYTIME);
     timeMI.addItemListener(command);
     timeMI.setState(config.display_time);
     configMenu.add(timeMI);

     addMI("Rename Title", configMenu, 
                           new CSPlane(this, CSPlane.RENAMETITLE));
     addMI("Resize Plot Area", configMenu,
                           new CSPlane(this, CSPlane.RESIZEPDA));

     Menu tickerMenu = new Menu("Tick Marks");
     tickerMI = new CheckboxMenuItem("Display Ticks");
     command = new CSPlane(this, CSPlane.TICKMARK);
     tickerMI.addItemListener(command);
     tickerMI.setState(config.tickmark);
     tickerMenu.add(tickerMI);

     ctickerMI = new MenuItem("Change Ticks");
     command = new CSPlane(this, CSPlane.TICKERCHANGE);
     ctickerMI.addActionListener(command);
     tickerMenu.add(ctickerMI);
     if (config.tickmark)
        ctickerMI.setEnabled(true);
     else
        ctickerMI.setEnabled(false);

     configMenu.add(tickerMenu);

     checkMI = new CheckboxMenuItem("Use global minmax");
     command = new CSPlane(this, CSPlane.GMINMAX);
     checkMI.addItemListener(command);
     checkMI.setState(config.global_mode);
     configMenu.add(checkMI);


     menubar.add(configMenu);
     /////////////////////////////////////////////////////////////////////
     /////////////////////////////////////////////////////////////////////
     // Misc menu
     Menu miscMenu = new Menu("Misc");

     // animation menus
     Menu animationMenu = new Menu("Animation");

     animateMI = new MenuItem("Animate C-a");
     command = new CSPlane(this, CSPlane.ANIMATE);
     animateMI.addActionListener(command);
     animationMenu.add(animateMI);

     stopMI = new MenuItem("Stop C-s");
     command = new CSPlane(this, CSPlane.STOP);
     stopMI.addActionListener(command);
     animationMenu.add(stopMI);
     stopMI.setEnabled(false);

     nextMI = new MenuItem("Next C-n");
     command = new CSPlane(this, CSPlane.NEXT);
     nextMI.addActionListener(command);
     animationMenu.add(nextMI);

     prevMI = new MenuItem("Previous C-p");
     command = new CSPlane(this, CSPlane.PREVIOUS);
     prevMI.addActionListener(command);
     animationMenu.add(prevMI);
     miscMenu.add(animationMenu);

     addMI("Buffer Size", miscMenu, 
                    new CSPlane(this, CommandSet.BUFFERSIZE));

     mi = new MenuItem("Goto C-g");
     command = new CSPlane(this, CSPlane.GOTO);
     mi.addActionListener(command);
     miscMenu.add(mi);

     // Split the DataSet
     splitMI = new MenuItem("Split DataSets");
     command = new CSPlane(this, CSPlane.SPLIT);
     splitMI.addActionListener(command);
     miscMenu.add(splitMI);
     if (dataSetVector.size() < 2) 
        splitMI.setEnabled(false);

     // Stealth mode
     stealthMI = new CheckboxMenuItem("Stealth Mode");
     command = new CSPlane(this, CSPlane.STEALTH);
     stealthMI.addItemListener(command);
     stealthMI.setState(stealthMode);
     miscMenu.add(stealthMI);

     // zooming
     Menu zoomMenu = new Menu("Zoom");

     Menu zoomModeMenu = new Menu("Mode");
     addMI("xy Zoom", zoomModeMenu, new CSPlane(this, CSPlane.XYZOOM));
     addMI("x Zoom", zoomModeMenu, new CSPlane(this, CSPlane.XZOOM));
     addMI("y Zoom", zoomModeMenu, new CSPlane(this, CSPlane.YZOOM));
     addMI("no Zoom", zoomModeMenu, new CSPlane(this, CSPlane.NOZOOM));
     zoomMenu.add(zoomModeMenu);

     zoomoutMI = new MenuItem("Zoomout C-o");
     command = new CSPlane(this, CSPlane.ZOOMOUT);
     zoomoutMI.addActionListener(command);
     zoomMenu.add(zoomoutMI);
     zoomoutMI.setEnabled(false);

     mi = new MenuItem("Reset");
     command = new CSPlane(this, CSPlane.RESET);
     mi.addActionListener(command);
     zoomMenu.add(mi);
     miscMenu.add(zoomMenu);
    
     menubar.add(miscMenu);

     setMenuBar(menubar);
  }

}  // end of class

